==> syncart-8.x-2.31.tar.gz: syncart/src/Service/SynCartService.php <==
<?php

namespace Drupal\syncart\Service;

use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_store\Entity\Store;
use Drupal\image\Entity\ImageStyle;

/**
 * Class SynCartService.
 */
class SynCartService implements SynCartServiceInterface {

  /**
   * Провайдер корзины.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * Manager корзины.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;


  /**
   * Объект текущей корзины.
   *
   * @var \Drupal\commerce_order\Entity\OrderInterface
   */
  protected $cart;

  /**
   * Constructs a new SynCartService object.
   */
  public function __construct(CartProviderInterface $cart_provider, CartManagerInterface $cart_manager) {
    $this->cartProvider = $cart_provider;
    $this->cartManager = $cart_manager;
    // Получение текущей корзины.
    $this->cart = $this->cartProvider->getCart('default');
    // Проверка на наличие дублирующих корзин.
    $this->clearCarts();
    $this->vid = 0;
  }

  /**
   * Получить текущую корзину пользователя.
   */
  public function load() {
    return $this->cart;
  }

  /**
   * Добавить товар в корзину.
   */
  public function addItem($vid, $quantity = 1) {
    if (empty($this->cart)) {
      $this->createCart();
    }
    $this->vid = $vid;
    $variation = \Drupal::entityTypeManager()
      ->getStorage('commerce_product_variation')
      ->load($vid);
    if (is_object($variation)) {
      $this->setOrderItem($variation, $quantity);
      $this->cart->recalculateTotalPrice();
      $this->cart->save();
    }
    return $this->cart;
  }

  /**
   * Удалить товар из корзины.
   */
  public function removeOrderItem($orderItemId) {
    $orderItem = OrderItem::load($orderItemId);
    if (is_object($orderItem)) {
      $this->cart->removeItem($orderItem);
      $this->cart->recalculateTotalPrice();
      return $this->cart->save();
    }
    return FALSE;
  }

  /**
   * Обновить количество товара в корзине.
   */
  public function setOrderItemQuantity($orderItemId, $quantity = 1) {
    if (empty($orderItemId) || !is_numeric($orderItemId)) {
      return FALSE;
    }
    if (is_object($orderItem = OrderItem::load($orderItemId))) {
      $orderItem->setQuantity($quantity);
      $this->cart->save();
      $this->cart->recalculateTotalPrice();
      return $this->cart->save();
    }
    return FALSE;
  }

  /**
   * Если в заказе есть вариация, увеличить кол-во, иначе добавить новую.
   */
  private function setOrderItem($variation, $quantity) {
    $hasItem = FALSE;
    if (!is_object($this->cart)) {
      return;
    }
    foreach ($this->cart->getItems() as $cartItem) {
      if ($cartItem->getPurchasedEntityId() == $this->vid) {
        $newQuantity = (int) ($cartItem->getQuantity() + $quantity);
        $cartItem->setQuantity($newQuantity);
        $cartItem->save();
        $hasItem = TRUE;
        break;
      }
    }
    // Если вариация ранее не была добавлена, добавляем.
    if (!$hasItem) {
      $orderItem = \Drupal::entityTypeManager()
        ->getStorage('commerce_order_item')
        ->createFromPurchasableEntity($variation, ['quantity' => $quantity]);
      if (is_object($orderItem)) {
        $this->cart->addItem($orderItem);
      }
    }
    $this->cart->save();
  }

  /**
   * Объединение и очистка дублирующихся корзин.
   */
  private function clearCarts() {
    $carts = $this->cartProvider->getCarts();
    if (count($carts) > 1) {
      $goods = [];
      foreach ($carts as $key => $otherCart) {
        if ($otherCart->id() !== $this->cart->id()) {
          foreach ($otherCart->getItems() as $cartItem) {
            $otherCart->removeItem($cartItem);
            $pid = $cartItem->purchased_entity->target_id;
            $quantity = $cartItem->quantity->value;
            if (isset($goods[$pid])) {
              $goods[$pid] += $quantity;
            }
            else {
              $goods[$pid] = $quantity;
            }
          }
          $otherCart->delete();
        }
      }
      foreach ($goods as $pid => $quantity) {
        $this->addItem($pid, $quantity);
      }
    }
  }

  /**
   * Собрать массив данных о текущей корзине для страницы корзины.
   */
  public function renderCartPageInfo($render = []) {
    $render = [
      'price' => 0,
      'number' => 0,
      'quantity' => 0,
    ];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
      foreach ($items as $index => $orderItem) {
        $variationInfo = $this->getVariationInfo($orderItem);
        if (empty($variationInfo)) {
          continue;
        }
        $render['id'] = $this->cart->id();
        $render['items'][$orderItem->id()] = $variationInfo;
        $render['quantity'] += $orderItem->getQuantity();
      }
      if (!empty($items)) {
        $price = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $this->cart->getTotalPrice(),
          ],
        ];
        $render['price'] = \Drupal::service('renderer')->renderRoot($price);
        $render['number'] = $this->cart->getTotalPrice()->getNumber();
      }
    }
    return $render;
  }

  /**
   * Получить информацию о вариации из товара в корзине.
   */
  private function getVariationInfo($orderItem, $info = []) {
    $currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    if (is_object($variation = $orderItem->getPurchasedEntity())) {
      $this->getVariationProductInfo($variation, $info);
      $currencyCode = $variation->getPrice()->getCurrencyCode();
      $info['id'] = $orderItem->id();
      $info['quantity'] = round($orderItem->getQuantity());
      $info['stock'] = intval($variation->field_stock->value);
      $info['number'] = $variation->getPrice()->getNumber();
      $price = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $variation->getPrice(),
        ],
      ];
      $total = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $orderItem->getTotalPrice(),
        ],
      ];
      $info['price'] = \Drupal::service('renderer')->renderRoot($price);
      $info['totalPrice'] = \Drupal::service('renderer')->renderRoot($total);
    }
    return $info;
  }

  /**
   * Получить информацию о товаре вариации.
   */
  private function getVariationProductInfo($variation, &$info) {
    if (is_object($product = $variation->getProduct())) {
      $title = $product->field_title->value;
      $info['title'] = $title ? $title : $product->getTitle();
      $info['picture'] = $this->getProductPicture($product);
      $info['url'] = $product->url();
      $info['pid'] = $product->id();
    }
  }

  /**
   * Загрузка изображения товара.
   */
  private function getProductPicture($product) {
    $image = '';
    $imageUri = '';
    $fieldImage = $product->field_image;
    $fieldGallery = $product->field_gallery;
    if (!$fieldImage->isEmpty()) {
      foreach ($fieldImage as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    elseif (!$fieldGallery->isEmpty()) {
      foreach ($fieldGallery as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    if ($imageUri) {
      $image = ImageStyle::load('product_cart')->buildUrl($imageUri);
    }
    return $image;
  }

  /**
   * Перевод корзины в статус заказа.
   */
  public function cartToOrder() {
    $this->cart->set('state', 'completed');
    $this->cart->save();
    return $this->cart;
  }

  /**
   * Создать новую корзину.
   */
  private function createCart() {
    $account = \Drupal::currentUser();
    $store = Store::load(1);
    $this->cart = $this->cartProvider->createCart('default', $store, $account);
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $items = [];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
    }
    return empty($items);
  }

  /**
   * {@inheritdoc}
   */
  public function sendReceipt($order) {
    $mail_handler = \Drupal::service('commerce.mail_handler');
    $order_total_summary = \Drupal::service('commerce_order.order_total_summary');
    $order_type_storage = \Drupal::entityTypeManager()->getStorage('commerce_order_type');
    $order_type = $order_type_storage->load($order->bundle());
    $profile = $order->getBillingProfile();
    if (!is_object($profile)) {
      return;
    }
    $to = $profile->field_customer_email->value;
    if (!$to) {
      return;
    }

    $profile_view_builder = \Drupal::entityTypeManager()->getViewBuilder('profile');
    $subject = t('Order #@number confirmed', ['@number' => $order->getOrderNumber()]);
    $body = [
      '#theme' => 'commerce_order_receipt',
      '#order_entity' => $order,
      '#totals' => $order_total_summary->buildTotals($order),
    ];
    $body['#billing_information'] = $profile_view_builder->view($profile);
    $params = [
      'id' => 'order_receipt',
      'from' => $order->getStore()->getEmail(),
      'bcc' => $order_type->getReceiptBcc(),
      'order' => $order,
    ];
    $customer = $order->getCustomer();
    if ($customer->isAuthenticated()) {
      $params['langcode'] = $customer->getPreferredLangcode();
    }
    $mail_handler->sendMail($to, $subject, $body, $params);
  }

  /**
   * Добавить заказ.
   */
  public function addOrderItem($commerce_order) {
    $this->order = $commerce_order;

    if (empty($this->cart)) {
      $this->createCart();
    }

    foreach ($this->order->getItems() as $order_item) {
      $variation = $order_item->getPurchasedEntity();
      $product = $variation ? $variation->getProduct() : NULL;
      if ($product && $product->isPublished()) {
        $order_item_new = $order_item->createDuplicate();
        $order_item_new->enforceIsNew();
        $order_item_new->id = NULL;
        $order_item_new->order_item_id = NULL;
        $order_item_new->save();
        $this->cartManager->addOrderItem($this->cart, $order_item_new);
      }
      else {
        $message = "Some products weren't copied to the cart as they aren't currently available.";
      }
    }

  }

}

==> syncart-8.x-2.32.tar.gz: syncart/src/Service/SynCartService.php <==
<?php

namespace Drupal\syncart\Service;

use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_store\Entity\Store;
use Drupal\image\Entity\ImageStyle;

/**
 * Class SynCartService.
 */
class SynCartService implements SynCartServiceInterface {

  /**
   * Провайдер корзины.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * Объект текущей корзины.
   *
   * @var \Drupal\commerce_order\Entity\OrderInterface
   */
  protected $cart;

  /**
   * Constructs a new SynCartService object.
   */
  public function __construct(CartProviderInterface $cart_provider) {
    $this->cartProvider = $cart_provider;
    // Получение текущей корзины.
    $this->cart = $this->cartProvider->getCart('default');
    // Проверка на наличие дублирующих корзин.
    $this->clearCarts();
    $this->vid = 0;
  }

  /**
   * Получить текущую корзину пользователя.
   */
  public function load() {
    return $this->cart;
  }

  /**
   * Добавить товар в корзину.
   */
  public function addItem($vid, $quantity = 1) {
    if (empty($this->cart)) {
      $this->createCart();
    }
    $this->vid = $vid;
    $variation = \Drupal::entityTypeManager()
      ->getStorage('commerce_product_variation')
      ->load($vid);
    if (is_object($variation)) {
      $this->setOrderItem($variation, $quantity);
      $this->cart->recalculateTotalPrice();
      $this->cart->save();
    }
    return $this->cart;
  }

  /**
   * Удалить товар из корзины.
   */
  public function removeOrderItem($orderItemId) {
    $orderItem = OrderItem::load($orderItemId);
    if (is_object($orderItem)) {
      $this->cart->removeItem($orderItem);
      $this->cart->recalculateTotalPrice();
      return $this->cart->save();
    }
    return FALSE;
  }

  /**
   * Обновить количество товара в корзине.
   */
  public function setOrderItemQuantity($orderItemId, $quantity = 1) {
    if (empty($orderItemId) || !is_numeric($orderItemId)) {
      return FALSE;
    }
    if (is_object($orderItem = OrderItem::load($orderItemId))) {
      $orderItem->setQuantity($quantity);
      $this->cart->save();
      $this->cart->recalculateTotalPrice();
      return $this->cart->save();
    }
    return FALSE;
  }

  /**
   * Если в заказе есть вариация, увеличить кол-во, иначе добавить новую.
   */
  private function setOrderItem($variation, $quantity) {
    $hasItem = FALSE;
    if (!is_object($this->cart)) {
      return;
    }
    foreach ($this->cart->getItems() as $cartItem) {
      if ($cartItem->getPurchasedEntityId() == $this->vid) {
        $newQuantity = (int) ($cartItem->getQuantity() + $quantity);
        $cartItem->setQuantity($newQuantity);
        $cartItem->save();
        $hasItem = TRUE;
        break;
      }
    }
    // Если вариация ранее не была добавлена, добавляем.
    if (!$hasItem) {
      $orderItem = \Drupal::entityTypeManager()
        ->getStorage('commerce_order_item')
        ->createFromPurchasableEntity($variation, ['quantity' => $quantity]);
      if (is_object($orderItem)) {
        $this->cart->addItem($orderItem);
      }
    }
    $this->cart->save();
  }

  /**
   * Объединение и очистка дублирующихся корзин.
   */
  private function clearCarts() {
    $carts = $this->cartProvider->getCarts();
    if (count($carts) > 1) {
      $goods = [];
      foreach ($carts as $key => $otherCart) {
        if ($otherCart->id() !== $this->cart->id()) {
          foreach ($otherCart->getItems() as $cartItem) {
            $otherCart->removeItem($cartItem);
            $pid = $cartItem->purchased_entity->target_id;
            $quantity = $cartItem->quantity->value;
            if (isset($goods[$pid])) {
              $goods[$pid] += $quantity;
            }
            else {
              $goods[$pid] = $quantity;
            }
          }
          $otherCart->delete();
        }
      }
      foreach ($goods as $pid => $quantity) {
        $this->addItem($pid, $quantity);
      }
    }
  }

  /**
   * Собрать массив данных о текущей корзине для страницы корзины.
   */
  public function renderCartPageInfo($render = []) {
    $render = [
      'price' => 0,
      'number' => 0,
      'quantity' => 0,
    ];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
      foreach ($items as $index => $orderItem) {
        $variationInfo = $this->getVariationInfo($orderItem);
        if (empty($variationInfo)) {
          continue;
        }
        $render['id'] = $this->cart->id();
        $render['items'][$orderItem->id()] = $variationInfo;
        $render['quantity'] += $orderItem->getQuantity();
      }
      if (!empty($items)) {
        $price = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $this->cart->getTotalPrice(),
          ],
        ];
        $render['price'] = \Drupal::service('renderer')->renderRoot($price);
        $render['number'] = $this->cart->getTotalPrice()->getNumber();
      }
    }
    return $render;
  }

  /**
   * Получить информацию о вариации из товара в корзине.
   */
  private function getVariationInfo($orderItem, $info = []) {
    $currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    if (is_object($variation = $orderItem->getPurchasedEntity())) {
      $this->getVariationProductInfo($variation, $info);
      $currencyCode = $variation->getPrice()->getCurrencyCode();
      $info['id'] = $orderItem->id();
      $info['quantity'] = round($orderItem->getQuantity());
      $info['stock'] = intval($variation->field_stock->value);
      $info['number'] = $variation->getPrice()->getNumber();
      $price = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $variation->getPrice(),
        ],
      ];
      $total = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $orderItem->getTotalPrice(),
        ],
      ];
      $info['price'] = \Drupal::service('renderer')->renderRoot($price);
      $info['totalPrice'] = \Drupal::service('renderer')->renderRoot($total);
    }
    return $info;
  }

  /**
   * Получить информацию о товаре вариации.
   */
  private function getVariationProductInfo($variation, &$info) {
    if (is_object($product = $variation->getProduct())) {
      $title = $product->field_title->value;
      $info['title'] = $title ? $title : $product->getTitle();
      $info['picture'] = $this->getProductPicture($product);
      $info['url'] = $product->url();
      $info['pid'] = $product->id();
    }
  }

  /**
   * Загрузка изображения товара.
   */
  private function getProductPicture($product) {
    $image = '';
    $imageUri = '';
    $fieldImage = $product->field_image;
    $fieldGallery = $product->field_gallery;
    if (!$fieldImage->isEmpty()) {
      foreach ($fieldImage as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    elseif (!$fieldGallery->isEmpty()) {
      foreach ($fieldGallery as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    if ($imageUri) {
      $image = ImageStyle::load('product_cart')->buildUrl($imageUri);
    }
    return $image;
  }

  /**
   * Перевод корзины в статус заказа.
   */
  public function cartToOrder() {
    $this->cart->set('state', 'completed');
    $this->cart->save();
    return $this->cart;
  }

  /**
   * Создать новую корзину.
   */
  private function createCart() {
    $account = \Drupal::currentUser();
    $store = Store::load(1);
    $this->cart = $this->cartProvider->createCart('default', $store, $account);
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $items = [];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
    }
    return empty($items);
  }

  /**
   * {@inheritdoc}
   */
  public function sendReceipt($order) {
    $mail_handler = \Drupal::service('commerce.mail_handler');
    $order_total_summary = \Drupal::service('commerce_order.order_total_summary');
    $order_type_storage = \Drupal::entityTypeManager()->getStorage('commerce_order_type');
    $order_type = $order_type_storage->load($order->bundle());
    $profile = $order->getBillingProfile();
    if (!is_object($profile)) {
      return;
    }
    $to = $profile->field_customer_email->value;
    if (!$to) {
      return;
    }

    $profile_view_builder = \Drupal::entityTypeManager()->getViewBuilder('profile');
    $subject = t('Order #@number confirmed', ['@number' => $order->getOrderNumber()]);
    $body = [
      '#theme' => 'commerce_order_receipt',
      '#order_entity' => $order,
      '#totals' => $order_total_summary->buildTotals($order),
    ];
    $body['#billing_information'] = $profile_view_builder->view($profile);
    $params = [
      'id' => 'order_receipt',
      'from' => $order->getStore()->getEmail(),
      'bcc' => $order_type->getReceiptBcc(),
      'order' => $order,
    ];
    $customer = $order->getCustomer();
    if ($customer->isAuthenticated()) {
      $params['langcode'] = $customer->getPreferredLangcode();
    }
    $mail_handler->sendMail($to, $subject, $body, $params);
  }

  /**
   * Добавить заказ.
   */
  public function addOrderItem($commerce_order) {
    $cart_manager = \Drupal::service('commerce_cart.cart_manager');
    $this->order = $commerce_order;

    if (empty($this->cart)) {
      $this->createCart();
    }

    foreach ($this->order->getItems() as $order_item) {
      $variation = $order_item->getPurchasedEntity();
      $product = $variation ? $variation->getProduct() : NULL;
      if ($product && $product->isPublished()) {
        $order_item_new = $order_item->createDuplicate();
        $order_item_new->enforceIsNew();
        $order_item_new->id = NULL;
        $order_item_new->order_item_id = NULL;
        $order_item_new->save();
        $cart_manager->addOrderItem($this->cart, $order_item_new);
      }
      else {
        $message = "Some products weren't copied to the cart as they aren't currently available.";
      }
    }

  }

}
